// *****************************************************************************
// CheckFieldData.cpp
//
// Used for comparing variant data types and ESRI field types
// *****************************************************************************

#include "CheckFieldData.h"

// *****************************************************************************
// NOTES:  Loops through the vectors of data and field types.  Checks to see
//         if the VARIANT data is compatible with the ESRI field type
// *****************************************************************************
bool CCheckFieldData::FieldDataMatches (VEC_VARIANT vData, VEC_FTYPE vTypes)
{
  int i;

  // Make sure the vectors are the same size
  if (vData.size() != vTypes.size())
    return false;

  for (i = 0; i < (int)vTypes.size(); ++i)
  {
    if (!Comp(vData[i], vTypes[i]))
      return false;
  }

  return true;
} // CCheckFieldData::FieldDataMatches

// *****************************************************************************
// NOTES:  Compares an individual VARTYPE with an esriFieldType.
// *****************************************************************************
bool CCheckFieldData::Comp (CComVariant val, esriFieldType type)
{
  bool rval = false;
  VARTYPE vt = val.vt;

  switch (type) {
    // Int types
    case esriFieldTypeSmallInteger:
    case esriFieldTypeInteger:
      if (vt == VT_I8 ||
          vt == VT_I4 ||
          vt == VT_UI1 ||
          vt == VT_I2 ||
          vt == VT_BOOL ||
          vt == VT_I1 ||
          vt == VT_UI2 ||
          vt == VT_UI4 ||
          vt == VT_UI2 ||
          vt == VT_INT ||
          vt == VT_UINT)
        rval = true;
      break;

    // Float types
    case esriFieldTypeSingle:
    case esriFieldTypeDouble:
      if (vt == VT_R4 ||
          vt == VT_R8)
        rval = true;
      break;

    // Strings
    case esriFieldTypeString:
      if (vt == VT_BSTR)
        rval = true;
      break;

    // Dates
    case esriFieldTypeDate:
      if (vt == VT_DATE)
        rval = true;
      break;

    // BLOB's
    case esriFieldTypeBlob:
      if (vt == VT_UNKNOWN ||
          vt == VT_ARRAY)
        rval = true;
      break;

    // Invalid -- should not occur
    case esriFieldTypeOID:
    case esriFieldTypeGeometry:
    case esriFieldTypeRaster:
    case esriFieldTypeGUID:
    case esriFieldTypeGlobalID:
    case esriFieldTypeXML:
    default:
      rval = false;
      break;
  }

  return rval;
} // CCheckFieldData::Comp