#include "stdafx.h"

#include "ReadGDB.h"

// *****************************************************************************
// NOTES:  BSTR conversion helper function
static char emptyString[] = "";

// *****************************************************************************
// NOTES:  Tool for converting CComBSTR to char*
// *****************************************************************************
char* _convert(CComBSTR& bsComBSTR)
{
  static _bstr_t bs;

  bs = _bstr_t(((BSTR)bsComBSTR));
  return ((char*)bs == NULL ? emptyString : (char*)bs);
}

// *****************************************************************************
CReadGDB::CReadGDB(void)
{
  // Register for COM to use the ESRI stuff
  ::CoInitialize(0);

  m_szFile = "";
}

// *****************************************************************************
CReadGDB::~CReadGDB(void)
{
  // Unregister COM
  ::CoUninitialize();
}

// *****************************************************************************
// NOTES:  Finds the table
// *****************************************************************************
HRESULT CReadGDB::ReadLastRowAsStrings (CComBSTR a_tablename,
                                        VEC_VARIANT &fieldnames,
                                        VEC_VARIANT &fieldvalues)
{
  HRESULT        hr;
  ITablePtr      ipTable;
  IWorkspacePtr  ipWorkspace;
  ICursorPtr     ipCursor;
  IRowPtr        ipCurRow, ipRow;
  IFieldsPtr     ipFields;
  IFieldPtr      ipField;
  long           i, numfields;
  CComBSTR       szTemp;
  CComVariant    vValue;

  // Open the Geodatabase, start editing
  hr = OpenGeodatabase(&ipWorkspace);
  if (FAILED(hr))
    return hr;

  // Find the table
  hr = FindTable(ipWorkspace, a_tablename, &ipTable);
  if (FAILED(hr))
    return hr;

  // Read the table, go to the last row, and get the field names and field
  // values of the row
  hr = ipTable->Search(NULL, VARIANT_FALSE, &ipCursor);
  hr = ipCursor->NextRow(&ipCurRow);

  // Get the last row
  while (ipCurRow != NULL) {
    ipRow = ipCurRow;
    hr = ipCursor->NextRow(&ipCurRow);
  }
  
  // If we have a row, get all the field names and values
  if (ipRow != NULL) {
    // Get the fields
    hr = ipRow->get_Fields(&ipFields);
    if (FAILED(hr))
      return hr;

    // Get the number of fields
    hr = ipFields->get_FieldCount(&numfields);
    if (FAILED(hr))
      return hr;
    for (i = 0; i < numfields; ++i) {
      // Get the current field
      hr = ipFields->get_Field(i, &ipField);
      if (FAILED(hr))
        return hr;
      if (ipField != NULL) {
        hr = ipField->get_Name(&szTemp);
        if (FAILED(hr)) return hr;

        // Store the field name
        fieldnames.push_back(szTemp);

        // Store the field value after converting it to a string
        hr = ipRow->get_Value(i, &vValue);
        if (FAILED(hr)) return hr;
        vValue.ChangeType(VT_BSTR); // Convert to string
        fieldvalues.push_back(vValue);
      }

    }
  }

  // Stop editing
  hr = StopEditing(ipWorkspace);
  if (FAILED(hr))
    return hr;

  return hr;
}

// *****************************************************************************
// NOTES:  Finishes editing the workspace (do not save edits!)
// *****************************************************************************
HRESULT CReadGDB::StopEditing (IWorkspacePtr ipWorkspace)
{
  HRESULT            hr;
  VARIANT_BOOL       bEditing;
  IWorkspaceEditPtr  ipWorkspaceEdit(ipWorkspace);

  // Check if we're editing right now
  hr = ipWorkspaceEdit->IsBeingEdited(&bEditing);

  // If editing, stop editing, not saving our edits (we should be just reading)
  if (bEditing == VARIANT_TRUE)
    hr = ipWorkspaceEdit->StopEditing(VARIANT_FALSE);

  return(hr);
}