#ifndef __CREATEGDB_H__
#define __CREATEGDB_H__

#include "ArcViewIncludes.h"
#include <atlbase.h>
#include "StdString.h"

////////////////////////////////////////////////////////////////////////////////
//// ma_tables enumeration
enum ma_tables {
               // Global Settings:
               MDFGLOBALS_TABLE = 1,

               // namefile
               NAMEFILE_TABLE,

               // Discretization (DIS):
               DELRC_TABLE, STRESSPERIODS_TABLE, DISLAYERS_TABLE,
               TOPELEV_TABLE, BOTMELEV_TABLE, DISVARS_TABLE,

               // Basic (BAS6):
               BASICVARS_TABLE, BASIC_TABLE, BASIC_MULT_TABLE,

               // Output Control (OC):
               OCVARS_TABLE, OCTS_TABLE, CBFLAGS_TABLE,

               // Block-Centered Flow (BCF6):
               BCFVARS_TABLE, BCFLAYERS_TABLE, BCFPROPERTIES_TABLE,

               // Layer-Property Flow (LPF):
               LPFVARS_TABLE, LPFLAYERS_TABLE, LPFPROPERTIES_TABLE,

               // List-Based Stress Packages:
               RIV_TABLE, CHD_TABLE, WEL_TABLE, DRN_TABLE,
               GHB_TABLE, HFB6_TABLE, DRT_TABLE,

               // MNW
               MNW1_TABLE, MNW1VARS_TABLE,

               // Recharge (RCH):
               RCHVARS_TABLE, RCHARRAYMULT_TABLE, RCHARRAYS_TABLE,

               // Evapotranspiration (ET):
               EVTVARS_TABLE, EVTARRAYMULT_TABLE, EVTARRAYS_TABLE,

               // Evapotranspiration Segments (ETS):
               ETSVARS_TABLE, ETSARRAYS_TABLE, ETSARRAYMULT_TABLE,
               ETSSEGARRAYS_TABLE, ETSSEGARRAYMULT_TABLE,

               // Solver:
               SIP_TABLE, PCG_TABLE, GMG_TABLE, DE4_TABLE, LMG_TABLE,
               SOR_TABLE,

               // Parameters
               PARAMS_TABLE, CLUSTERS_TABLE, IZ_TABLE,
               PILOTPTS_TABLE, MULTNAMES_TABLE, MULTIPLIERS_TABLE,
               ZONES_TABLE, ZONENAMES_TABLE, PARINSTANCES_TABLE,
               PARINSTSP_TABLE,

               // Observations
               OBSVARS_TABLE, HOB_TABLE, HOBLAYERS_TABLE, HOBTIMES_TABLE,
               FLOB_TABLE, FLOBFACTORS_TABLE,

               // SEN, PES Processes
               SEN_TABLE, PES_TABLE,

               // Output
               OUTPUTFILES_TABLE, OUTPUTTIME_TABLE, OUTPUTHEAD_TABLE,
               OUTPUTDRAWDOWN_TABLE, OUTPUTFLOW_TABLE,

               // Hydrogeologic Unit Flow (HUF2)
               HUFVARS_TABLE, HUFLAYERS_TABLE, HUFWETDRY_TABLE,
               HUFUNITS_TABLE, HUFUNITPROPS_TABLE,

               // Stream (STR6)
               STRVARS_TABLE, STRITRIB_TABLE, STRIUPSEG_TABLE, STRREACHES_TABLE,

               // Streamflow-Routing (SFR2)
               SFRVARS_TABLE, SFRREACHES_TABLE, SFRFLOWTAB_TABLE,
               SFRSEGMENTS_TABLE, SFRXSECT_TABLE,

               // Gaging Station File (GAGE)
               GAGSFR_TABLE, GAGLAK_TABLE,

               // Lake (LAK3)
               LAKVARS_TABLE, LAKARRAYS_TABLE, LAKSTAGES_TABLE,
               LAKSUBLAKES_TABLE, LAKRATES_TABLE,

               // Unsaturated Zone Flow (UZF)
               UZFVARS_TABLE, UZFARRAYS_TABLE, UZFARRAYMULT_TABLE,
               UZFGAGES_TABLE, UZFSTRESSARRAYS_TABLE, UZFSTRESSARRAYMULT_TABLE,

               MATABLES_END // Leave this at the end
               };

////////////////////////////////////////////////////////////////////////////////
//// CCreateGDB
class CCreateGDB
{
  public:
    CCreateGDB(void);
    ~CCreateGDB(void);

  public:
    void    SetGDBFile (CComBSTR a_path, CComBSTR a_name, int a_type)
                       { m_szFile = a_name;
                         m_szPath = a_path;
                         m_Personal = a_type; }
    void    SetGDBFile2 (CComBSTR a_pathandfile);
    void    SetGDBFile3 (const std::string& a_path, const std::string& a_file)
                        {
                          m_szStdPath = a_path;
                          m_szStdFile = a_file; }
    HRESULT SetupNewGDB (bool a_CreateAllModflowTables=false);
    void    DEBUG_DELETE (IFeatureWorkspacePtr ipFW);

  protected:
    CComBSTR m_szPath;
    CComBSTR m_szFile;
    std::string m_szStdPath;
    std::string m_szStdFile;
    int      m_Personal;

  protected:
    HRESULT CreateField (CComBSTR szName, esriFieldType eType, bool bNullable,
                         long lPrecision, long lLength, long lDefault,
                         long lScale, IDomainPtr ipDomain, IFieldEdit **ppFE);
    HRESULT CreateTableUID (IUID **ppUID, IUID **ppEXTUID);
    HRESULT CreateFeatureUID (IUID **ppUID);
    HRESULT AddModflowFieldsToFieldsEdit (ma_tables ftype, IFieldsEditPtr ipFE);
    bool    FileExists (const std::string& strFilename);

  public:
    // Workspace related functions
    HRESULT CreateWorkspaceFactory (IWorkspaceFactory **ppFact);
    HRESULT CreateWorkspace (IWorkspaceFactoryPtr ipFact,
                             IWorkspace **ppWorkspace);
    HRESULT StartEditing (IWorkspacePtr ipWorkspace);
    HRESULT StopEditing (IWorkspacePtr ipWorkspace);

    // Field and field editing functions
    HRESULT CreateFieldsEditFeature (esriGeometryType geom, IFieldsEdit **ppFE,
                                     ISpatialReferencePtr ipSRef = 0,
                                     bool bHasZ = false, bool bHasM = true);
    HRESULT CreateFieldsEditTable (IFieldsEdit **ppFE);
    HRESULT AddFieldToFieldsEdit (IFieldsEditPtr ipFE, CComBSTR name,
                                  esriFieldType type, VARIANT_BOOL isNullable);

    // Table and table editing functions
    HRESULT CreateNewTable (CComBSTR szName, IFeatureWorkspacePtr ipFW,
                            IFieldsEditPtr ipFields, IUIDPtr ipUID,
                            IUIDPtr ipEXTUID, ITable **ppTable);
    HRESULT AddAllModflowTables (IFeatureWorkspacePtr ipFW);
    HRESULT AddAllOuputputTables (IFeatureWorkspacePtr ipFW);
    HRESULT AddAllParamTables (IFeatureWorkspacePtr ipFW);
    HRESULT AddAllObservationTables (IFeatureWorkspacePtr ipFW);
    bool    TableExists (IFeatureWorkspacePtr ipFW, CComBSTR szName);

    // Feature class and feature class editing functions
    HRESULT CreateNewFeatureClass (CComBSTR szName, IFeatureDatasetPtr ipFD,
                                   IFieldsEditPtr ipFields, IUIDPtr ipUID,
                                   IFeatureClass **ppFC);
    HRESULT CreateNewFeatureClassWksp (CComBSTR szName,
                                       IFeatureWorkspacePtr ipFW,
                                       IFieldsEditPtr ipFields, IUIDPtr ipUID,
                                       IFeatureClass **ppFC);
    HRESULT CreateBasicFeatureClass (CComBSTR szName, esriGeometryType geom,
                                     IFeatureDatasetPtr ipFeatureDataset,
                                     IUIDPtr ipUID, IFeatureClass **ipFC);
    HRESULT CreateBasicFeatureClassWksp (CComBSTR szName, esriGeometryType geom,
                                         IFeatureWorkspacePtr ipWorkspace,
                                         IUIDPtr ipUID, IFeatureClass **ipFC);

  // Adding MODFLOW tables
  public:
    // Generic Table:
    HRESULT AddTable (ma_tables TableID, IFeatureWorkspacePtr ipFW,
                      ITable **ppTable);

  protected:
    HRESULT AddGridFeatures (IFeatureClassPtr ipFC, int Instance);
    HRESULT AddNodeFeatures (IFeatureClassPtr ipFC, int Instance);

};

#endif