// *****************************************************************************
// NAME:     MDMDomains.cpp
//
// PURPOSE:  For managing and creating domains (coded value) for the
//           MODFLOW Data Model
// *****************************************************************************

//#include "stdafx.h"

#include <atlbase.h>
#include "MDMDomains.h"

//------------------------------------------------------------------------------
/// \brief constructor.
//------------------------------------------------------------------------------
CMDMDomains::CMDMDomains ()
: Singleton(),
  m_ipTimeUnitDomain(NULL),
  m_ipLengthUnitDomain(NULL),
  m_ipDirectionDomain(NULL),
  m_ipSteadyStateDomain(NULL),
  m_ipBCFMeanDomain(NULL),
  m_ipBCFLayerTypeDomain(NULL),
  m_ipLPFLayerTypeDomain(NULL),
  m_ipLPFMeanDomain(NULL),
  m_ipRechargeOptionDomain(NULL),
  m_ipEvapotranspirationOptionDomain(NULL),
  m_ipSTATFLAGDomain(NULL),
  m_ipFLOBTypeDomain(NULL),
  m_ipOutputFormatDomain(NULL),
  m_ipOutputFlowTypeDomain(NULL),
  m_ipPCGConvergenceDomain(NULL),
  m_ipLMGPrintDomain(NULL),
  m_ipGMGSemicoarseningDomain(NULL),
  m_ipGMGOutputDomain(NULL),
  m_ipOCVarsFileFormatDomain(NULL)
{
}
//------------------------------------------------------------------------------
/// \brief destructor.
//------------------------------------------------------------------------------
CMDMDomains::~CMDMDomains ()
{
}
// *****************************************************************************
// PURPOSE:  Creates all of the domains for a workspace (geodatabase)
//           Calls functions for the individual domains.
// *****************************************************************************
HRESULT CMDMDomains::CreateAHGWDomains (IWorkspacePtr ipWS)
{
  HRESULT               hr = S_OK;

  if (S_OK == CreateAHGWDomains())
  {
    hr = CMDMDomains::AddAllAHGWDomains(ipWS);
  }

  return hr;
} // CMDMDomains::CreateAHGWDomains
//----- OVERLOAD ---------------------------------------------------------------
HRESULT CMDMDomains::CreateAHGWDomains ()
{
  // Create all of the domains
  if (SUCCEEDED(CreateTimeUnitDomain(&m_ipTimeUnitDomain)) &&
      SUCCEEDED(CreateLengthUnitDomain(&m_ipLengthUnitDomain)) &&
      SUCCEEDED(CreateDirectionDomain(&m_ipDirectionDomain)) &&
      SUCCEEDED(CreateSteadyStateDomain(&m_ipSteadyStateDomain)) &&
      SUCCEEDED(CreateBCFMeanDomain(&m_ipBCFMeanDomain)) &&
      SUCCEEDED(CreateBCFLayerTypeDomain(&m_ipBCFLayerTypeDomain)) &&
      SUCCEEDED(CreateLPFLayerTypeDomain(&m_ipLPFLayerTypeDomain)) &&
      SUCCEEDED(CreateLPFMeanDomain(&m_ipLPFMeanDomain)) &&
      SUCCEEDED(CreateRechargeOptionDomain(&m_ipRechargeOptionDomain)) &&
      SUCCEEDED(CreateEvapotranspirationOptionDomain(&m_ipEvapotranspirationOptionDomain)) &&
      SUCCEEDED(CreateSTATFLAGDomain(&m_ipSTATFLAGDomain)) &&
      SUCCEEDED(CreateFLOBTypeDomain(&m_ipFLOBTypeDomain)) &&
      SUCCEEDED(CreateOutputFlowTypeDomain(&m_ipOutputFlowTypeDomain)) &&
      SUCCEEDED(CreatePCGConvergenceDomain(&m_ipPCGConvergenceDomain)) &&
      SUCCEEDED(CreateLMGPrintDomain(&m_ipLMGPrintDomain)) &&
      SUCCEEDED(CreateGMGSemicoarseningDomain(&m_ipGMGSemicoarseningDomain)) &&
      SUCCEEDED(CreateGMGOutputDomain(&m_ipGMGOutputDomain)) &&
      SUCCEEDED(CreateOCVarsFileFormatDomain(&m_ipOCVarsFileFormatDomain)))
  {
    return S_OK;
  }
  return S_FALSE;
} // CMDMDomains::CreateAHGWDomains
// *****************************************************************************
// PURPOSE:  Creates all of the domains for a workspace (geodatabase)
//           Calls functions for the individual domains.
// *****************************************************************************
HRESULT CMDMDomains::AddAllAHGWDomains (IWorkspacePtr ipWS)
{
  HRESULT               hr = S_OK;
  long                  lID;
  IWorkspaceDomainsPtr  ipWSDomains(ipWS);


  hr = ipWSDomains->AddDomain(m_ipTimeUnitDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipLengthUnitDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipDirectionDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipSteadyStateDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipBCFMeanDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipBCFLayerTypeDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipLPFLayerTypeDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipLPFMeanDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipRechargeOptionDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipEvapotranspirationOptionDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipSTATFLAGDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipFLOBTypeDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipOutputFlowTypeDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipPCGConvergenceDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipLMGPrintDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipGMGSemicoarseningDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipGMGOutputDomain, &lID);
  if (FAILED(hr)) return hr;

  hr = ipWSDomains->AddDomain(m_ipOCVarsFileFormatDomain, &lID);
  if (FAILED(hr)) return hr;

  return hr;
} // CMDMDomains::AddAllAHGWDomains
// *****************************************************************************
// PURPOSE:  Get a Coded Value Domain
// *****************************************************************************
IDomainPtr CMDMDomains::GetDomain ()
{
  return m_ipOutputFlowTypeDomain;
} // CMDMDomains::GetDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Time Unit
// *****************************************************************************
HRESULT CMDMDomains::CreateTimeUnitDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("undefined"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("seconds"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("minutes"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("hours"));
  vValue = 4;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("days"));
  vValue = 5;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("years"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("TimeUnit"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("MODFLOW Time Unit (ITMUNI)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateTimeUnitDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Length Unit
// *****************************************************************************
HRESULT CMDMDomains::CreateLengthUnitDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("undefined"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("feet"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("meters"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("centimeters"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("LengthUnit"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("MODFLOW Length Unit (LENUNI)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateLengthUnitDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the DELRC direction
// *****************************************************************************
HRESULT CMDMDomains::CreateDirectionDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = CComBSTR("R");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Row"));
  vValue = CComBSTR("C");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Column"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("Direction"));
  hr = ipDomain->put_FieldType(esriFieldTypeString);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDuplicate);
  hr = ipDomain->put_Description(CComBSTR("Discretization direction"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateDirectionDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Steady State flag
// *****************************************************************************
HRESULT CMDMDomains::CreateSteadyStateDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = CComBSTR("SS");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("SteadyState"));
  vValue = CComBSTR("Transient");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Transient"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("SteadyState"));
  hr = ipDomain->put_FieldType(esriFieldTypeString);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDuplicate);
  hr = ipDomain->put_Description(CComBSTR("Steady State or Transient"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateSteadyStateDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the BCF Mean flag
// *****************************************************************************
HRESULT CMDMDomains::CreateBCFMeanDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Harmonic mean (0)"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Arithmetic mean (1)"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Logarithmic mean (2)"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Arith t X log(k) (3)"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("BCFMean"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("BCF mean (LAYAVG)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateBCFMeanDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the BCF Layer Type flag
// *****************************************************************************
HRESULT CMDMDomains::CreateBCFLayerTypeDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Confined (0)"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Unconfined (1)"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Confined/Unconfined, T. Const. (2)"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Confined/Unconfined, T. Var. (3)"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("BCFLayerType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("BCF layer type (LAYCON)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateBCFLayerTypeDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the LPF Layer Type flag
// *****************************************************************************
HRESULT CMDMDomains::CreateLPFLayerTypeDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Confined"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Convertible"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("LPFLayerType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("LPF layer type (LAYTYP)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateLPFLayerTypeDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the LPF Mean flag
// *****************************************************************************
HRESULT CMDMDomains::CreateLPFMeanDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Harmonic mean (0)"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Arithmetic mean (1)"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Logarithmic mean (2)"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Arith t X log(k) (3)"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("LPFMean"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("LPF Mean (LAYAVG)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateLPFMeanDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Recharge Option flag
// *****************************************************************************
HRESULT CMDMDomains::CreateRechargeOptionDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Top"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Specified"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Highest Active"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("RechargeOption"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("Recharge option (NRCHOP)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateRechargeOptionDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Evapotranspiration Option flag
// *****************************************************************************
HRESULT CMDMDomains::CreateEvapotranspirationOptionDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Top"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Specified"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("EvapotranspirationOption"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("Evaporation option (NEVTOP)"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateEvapotranspirationOptionDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the HOBTimes STATFLAG flag
// *****************************************************************************
HRESULT CMDMDomains::CreateSTATFLAGDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Variance [L^2] (0)"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Std. Dev. [L] (1)"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Coeff. of variation [--] (3)"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("STATFLAG"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("Statistic"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateSTATFLAGDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the FLOB Type flag
// *****************************************************************************
HRESULT CMDMDomains::CreateFLOBTypeDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = CComBSTR("GBOB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("GBOB"));
  vValue = CComBSTR("DROB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("DROB"));
  vValue = CComBSTR("RVOB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("RVOB"));
  vValue = CComBSTR("CHOB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("CHOB"));
  vValue = CComBSTR("STOB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("STOB"));
  vValue = CComBSTR("DTOB");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("DTOB"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("FLOBType"));
  hr = ipDomain->put_FieldType(esriFieldTypeString);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDuplicate);
  hr = ipDomain->put_Description(CComBSTR("FLOBType"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateFLOBTypeDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Output Format type
// *****************************************************************************
HRESULT CMDMDomains::CreateOutputFormatDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = CComBSTR("binary");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("binary"));
  vValue = CComBSTR("ASCII");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("ASCII"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("OutputFormat"));
  hr = ipDomain->put_FieldType(esriFieldTypeString);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDuplicate);
  hr = ipDomain->put_Description(CComBSTR("File format for output"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateOutputFormatDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the Output Flow type
// *****************************************************************************
HRESULT CMDMDomains::CreateOutputFlowTypeDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = CComBSTR("Right");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Right"));
  vValue = CComBSTR("Front");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Front"));
  vValue = CComBSTR("Lower");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Lower"));
  vValue = CComBSTR("Wells");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Wells"));
  vValue = CComBSTR("Drains");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Drains"));
  vValue = CComBSTR("Rivers");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Rivers"));
  vValue = CComBSTR("ET");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("ET"));
  vValue = CComBSTR("General Heads");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("General Heads"));
  vValue = CComBSTR("Recharge");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Recharge"));
  vValue = CComBSTR("Constant Heads");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Constant Heads"));
  vValue = CComBSTR("Streams");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Streams"));
  vValue = CComBSTR("Storage");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Storage"));
  vValue = CComBSTR("Lakes");
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Lakes"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("OutputFlowType"));
  hr = ipDomain->put_FieldType(esriFieldTypeString);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDuplicate);
  hr = ipDomain->put_Description(CComBSTR("Output flow type"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateOutputFlowTypeDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the PCG Convergence
// *****************************************************************************
HRESULT CMDMDomains::CreatePCGConvergenceDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Print max head change and residual each iteration"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Print total number of iterations"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("No printing"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Print only if convergence fails"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("MUTPCGType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("PCG convergence information printing flag"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreatePCGConvergenceDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the LMG Print Flag IOUTAMG
// *****************************************************************************
HRESULT CMDMDomains::CreateLMGPrintDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("No printing"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Scaling and residuals"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Scaling, residuals, storage, time"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Solver, scaling, residuals, storage, time"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("IOUTAMGType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("LMG print flag"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateLMGPrintDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the GMG Semicoarsening flag ISC
// *****************************************************************************
HRESULT CMDMDomains::CreateGMGSemicoarseningDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Rows, columns, layers"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Rows, columns"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Columns, layers"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Rows, layers"));
  vValue = 4;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("No coarsening"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("ISCType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("GMG flag controlling semicoarsneing"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateGMGSemicoarseningDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the GMG Output flag IOUTGMG
// *****************************************************************************
HRESULT CMDMDomains::CreateGMGOutputDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 0;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Solver inputs"));
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Linear solve"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Convergence history"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Linear solve terminal output"));
  vValue = 4;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("Convergence history terminal output"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("IOUTGMGType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("GMG flag controlling output"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateGMGOutputDomain

// *****************************************************************************
// PURPOSE:  Creates the Coded Value Domain for the OCVars File Format (IHEDFM, IDDNFM)
// *****************************************************************************
HRESULT CMDMDomains::CreateOCVarsFileFormatDomain (IDomain **ppDomain)
{
  HRESULT               hr = S_OK;
  IDomainPtr            ipDomain;
  ICodedValueDomainPtr  ipCVDomain;
  CComVariant           vValue;

  // Create the domain
  hr = ipCVDomain.CreateInstance(CLSID_CodedValueDomain);
  if (FAILED(hr)) return hr;

  // Add the coded values to the domain
  vValue = 1;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("11G10.3"));
  vValue = 2;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("9G13.6"));
  vValue = 3;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("15F7.1"));
  vValue = 4;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("15F7.2"));
  vValue = 5;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("15F7.3"));
  vValue = 6;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("15F7.4"));
  vValue = 7;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("20F5.0"));
  vValue = 8;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("20F5.1"));
  vValue = 9;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("20F5.2"));
  vValue = 10;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("20F5.3"));
  vValue = 11;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("20F5.4"));
  vValue = 12;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10G11.4"));
  vValue = 13;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.0"));
  vValue = 14;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.1"));
  vValue = 15;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.2"));
  vValue = 16;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.3"));
  vValue = 17;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.4"));
  vValue = 18;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("10F6.5"));
  vValue = 19;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("5G12.5"));
  vValue = 20;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("6G11.4"));
  vValue = 21;  hr = vValue.ChangeType(VT_I2);
  hr = ipCVDomain->AddCode(vValue, CComBSTR("7G9.2"));

  // Set up the other domain parameters
  ipDomain = ipCVDomain;
  hr = ipDomain->put_Name(CComBSTR("OCVarsFileType"));
  hr = ipDomain->put_FieldType(esriFieldTypeSmallInteger);
  hr = ipDomain->put_MergePolicy(esriMPTDefaultValue);
  hr = ipDomain->put_SplitPolicy(esriSPTDefaultValue);
  hr = ipDomain->put_Description(CComBSTR("Output control file format"));

  // Add a reference to the domain
  (*ppDomain) = ipDomain;
  if (*ppDomain)
    (*ppDomain)->AddRef();

  return hr;
} // CMDMDomains::CreateOCVarsFileFormatDomain

#ifdef CXX_TEST

#include <MDMDomains.t.h>

////////////////////////////////////////////////////////////////////////////////
//
// class CMDMDomainsT
//
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
void CMDMDomainsT::testGetDomain ()
{
  //TS_FAIL("CMDMDomainsT::testGetDomain");

  // These tests fail. I'm guessing we have to initialize ArcObjects and stuff
  // to create the domains.

  //TS_ASSERT_EQUALS(CMDMDomains::Instance().CreateAHGWDomains(), S_OK);
  //TS_ASSERT(CMDMDomains::Instance().GetDomain() != NULL);
}

#endif

