#ifndef __MODFLOW_DATA_MODEL_DOMAINS_H__
#define __MODFLOW_DATA_MODEL_DOMAINS_H__

#include "ArcViewIncludes.h"
#include "singleton.h"

class CMDMDomains : public Singleton<CMDMDomains>
{
  friend Singleton<CMDMDomains>;

public:
  HRESULT CreateAHGWDomains(IWorkspacePtr ipWS);
  HRESULT CreateAHGWDomains();
  IDomainPtr GetDomain();

private:
  CMDMDomains();
  CMDMDomains(const CMDMDomains &rhs); // Copy Constructor 
  virtual ~CMDMDomains();              // Destructor
  const CMDMDomains& operator= (const CMDMDomains &rhs);

protected:
  HRESULT AddAllAHGWDomains(IWorkspacePtr ipWS);

  HRESULT CreateTimeUnitDomain (IDomain **ppDomain);
  HRESULT CreateLengthUnitDomain (IDomain **ppDomain);
  HRESULT CreateDirectionDomain (IDomain **ppDomain);
  HRESULT CreateSteadyStateDomain (IDomain **ppDomain);
  HRESULT CreateBCFMeanDomain (IDomain **ppDomain);
  HRESULT CreateBCFLayerTypeDomain (IDomain **ppDomain);
  HRESULT CreateLPFLayerTypeDomain (IDomain **ppDomain);
  HRESULT CreateLPFMeanDomain (IDomain **ppDomain);
  HRESULT CreateRechargeOptionDomain (IDomain **ppDomain);
  HRESULT CreateEvapotranspirationOptionDomain (IDomain **ppDomain);
  HRESULT CreateSTATFLAGDomain (IDomain **ppDomain);
  HRESULT CreateFLOBTypeDomain (IDomain **ppDomain);
  HRESULT CreateOutputFormatDomain (IDomain **ppDomain);
  HRESULT CreateOutputFlowTypeDomain (IDomain **ppDomain);
  HRESULT CreatePCGConvergenceDomain (IDomain **ppDomain);
  HRESULT CreateLMGPrintDomain (IDomain **ppDomain);
  HRESULT CreateGMGSemicoarseningDomain (IDomain **ppDomain);
  HRESULT CreateGMGOutputDomain (IDomain **ppDomain);
  HRESULT CreateOCVarsFileFormatDomain (IDomain **ppDomain);

public:
  IDomainPtr GetTimeUnitDomain () { return m_ipTimeUnitDomain; };
  IDomainPtr GetLengthUnitDomain () { return m_ipLengthUnitDomain; };
  IDomainPtr GetDirectionDomain () { return m_ipDirectionDomain; };
  IDomainPtr GetSteadyStateDomain () { return m_ipSteadyStateDomain; };
  IDomainPtr GetBCFMeanDomain () { return m_ipBCFMeanDomain; };
  IDomainPtr GetBCFLayerTypeDomain () { return m_ipBCFLayerTypeDomain; };
  IDomainPtr GetLPFLayerTypeDomain () { return m_ipLPFLayerTypeDomain; };
  IDomainPtr GetLPFMeanDomain () { return m_ipLPFMeanDomain; };
  IDomainPtr GetRechargeOptionDomain () { return m_ipRechargeOptionDomain; };
  IDomainPtr GetEvapotranspirationOptionDomain () { return m_ipEvapotranspirationOptionDomain; };
  IDomainPtr GetSTATFLAGDomain () { return m_ipSTATFLAGDomain; };
  IDomainPtr GetFLOBTypeDomain () { return m_ipFLOBTypeDomain; };
  IDomainPtr GetOutputFormatDomain () { return m_ipOutputFormatDomain; };
  IDomainPtr GetOutputFlowTypeDomain () { return m_ipOutputFlowTypeDomain; };
  IDomainPtr GetPCGConvergenceDomain () { return m_ipPCGConvergenceDomain; };
  IDomainPtr GetLMGPrintDomain () { return m_ipLMGPrintDomain; };
  IDomainPtr GetGMGSemicoarseningDomain () { return m_ipGMGSemicoarseningDomain; };
  IDomainPtr GetGMGOutputDomain () { return m_ipGMGOutputDomain; };
  IDomainPtr GetOCVarsFileFormatDomain () { return m_ipOCVarsFileFormatDomain; };

protected:
  IDomainPtr m_ipTimeUnitDomain;
  IDomainPtr m_ipLengthUnitDomain;
  IDomainPtr m_ipDirectionDomain;
  IDomainPtr m_ipSteadyStateDomain;
  IDomainPtr m_ipBCFMeanDomain;
  IDomainPtr m_ipBCFLayerTypeDomain;
  IDomainPtr m_ipLPFLayerTypeDomain;
  IDomainPtr m_ipLPFMeanDomain;
  IDomainPtr m_ipRechargeOptionDomain;
  IDomainPtr m_ipEvapotranspirationOptionDomain;
  IDomainPtr m_ipSTATFLAGDomain;
  IDomainPtr m_ipFLOBTypeDomain;
  IDomainPtr m_ipOutputFormatDomain;
  IDomainPtr m_ipOutputFlowTypeDomain;
  IDomainPtr m_ipPCGConvergenceDomain;
  IDomainPtr m_ipLMGPrintDomain;
  IDomainPtr m_ipGMGSemicoarseningDomain;
  IDomainPtr m_ipGMGOutputDomain;
  IDomainPtr m_ipOCVarsFileFormatDomain;
};
#endif