// ModflowLink.cpp : Defines the entry point for the DLL application.
//
#include<map>
#include <sstream>

#include "ModflowLink.h"
#include "EditGDB.h"
#include "CreateGDB.h"
#include "StdString.h"
#include "TableDef.h"
#include <DataToSafeArray.h>
#include <Grid.h>

#include <CppSQLite3.h>
#include <CEditSQLiteDb.h>


struct TableInfo
{
private:

protected:
  static TableInfo** GetInternal ()
  {
    static TableInfo *m_p(0);
    return &m_p;
  }

public:
  TableInfo () : m_tabId(-1) {}

  static bool Exists ()
  {
    return (*GetInternal() ? 1 : 0);
  }

  static TableInfo *Get()
  {
    if (!*GetInternal())
      *GetInternal() = new TableInfo();
    return *GetInternal();
  }
  static void Destroy()
  {
    if (*GetInternal())
      delete(*GetInternal());
    *GetInternal() = NULL;
  }

  virtual int WriteToDb(CComSafeArray<VARIANT> *valsPtr);

  CStr                   m_fname;
  CStr                   m_query;
  CComSafeArray<VARIANT> m_vals;
  CComSafeArray<BSTR>    m_fields;
  CComBSTR               m_tabName;
  std::map<CStr, UINT>   m_fieldIdx;
  int                    m_tabId;
};

struct TableInfoTester : public TableInfo
{
private:
public:
  static void MakeOne()
  {
    *GetInternal() = new TableInfoTester();
  }
  virtual int WriteToDb(CComSafeArray<VARIANT> *valsPtr) {return 1;}
};


template <class T>
static int iAddDataToCacheT(const char *a_field,
                            T a_val,
                            int a_row=-1);

#ifdef _MANAGED
#pragma managed(push, off)
#endif


BOOL APIENTRY DllMain (HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved)
{
	switch (ul_reason_for_call)	{
	  case DLL_PROCESS_ATTACH:
	  case DLL_THREAD_ATTACH:
	  case DLL_THREAD_DETACH:
	  case DLL_PROCESS_DETACH:
		  break;
	}

  return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

static int GetNumSP (int a_=-1)
{
  static int m_sp(0);
  if (a_!=-1 && a_ > 0)
    m_sp = a_;

  return (m_sp);
}

#if CXX_TEST
#include <RunTest.h>
#include <iostream>
#include <sstream>
int main (int argc, const char **argv)
{
  //cxxTest_RunAllTests() = true;
  testCxx::RunUnitTests();
  return(0);
} // main
#endif

static bool& UseSQLite ()
{
  static bool m_useSQLite(false);
  return m_useSQLite;
} // UseSQLite
static CppSQLite3DB& SQLiteDb ()
{
  static CppSQLite3DB m_db;
  return m_db;
} // SQLiteDb
// *****************************************************************************
// NOTES:  Creates a new geodatabase, with every single table
//
//         path:  The path where the geodatabase will be created
//         filename:  The filename of the new geodatabase
//         personal:  1 if you want a personal geodatabase, 0 for a file gdb
// *****************************************************************************
DLLEXPORT int fnCreateEmptyGDB (const char *path,
                                const char *filename,
                                const int *personal)
{
  HRESULT     hr;
  CCreateGDB  create;

  create.SetGDBFile(path, filename, *personal);
  create.SetGDBFile3(path, filename);

  hr = create.SetupNewGDB();

  if (FAILED(hr))
    return 0;

  return 1;
} // fnCreateEmptyGDB
DLLEXPORT int fnOpenExistingGDB (const char* path,
                                 const char* filename)
{
  CCreateGDB  create;
  create.SetGDBFile3(path, filename);

  HRESULT   hr;
  CEditGDB  edit;
  CStr fname(path);
  fname += filename;
  edit.SetGDBFile(fname.c_str());
  IWorkspacePtr  ipWorkspace;
  hr = edit.OpenGeodatabase2(&ipWorkspace);
  if (FAILED(hr)) return hr;
  hr = create.AddAllOuputputTables(ipWorkspace);
  if (FAILED(hr)) return hr;
  hr = create.AddAllParamTables(ipWorkspace);
  if (FAILED(hr)) return hr;
  hr = create.AddAllObservationTables(ipWorkspace);
  if (FAILED(hr)) return hr;

  //HRESULT     hr;
  //hr = create.SetupNewGDB();
  //if (FAILED(hr))
  //  return 0;

  return 1;
}
DLLEXPORT int fnCreateEmptySQLiteDB (const char* path,
                                     const char* filename,
                                     const int*  personal)
{
  std::string str(path);
  str += "\\";
  str += filename;
  SQLiteDb().open(str.c_str());
  CEditSQLiteDb sq(&SQLiteDb());
  SQLiteDb().execDML("begin transaction;");
  sq.SetupNewGDB();
  UseSQLite() = true;
  return 1;
} // fnCreateEmptySQLiteDB
DLLEXPORT int fnOpenExistingSQLite (const char* path,
                                    const char* filename)
{
  std::string str(path);
  str += "\\";
  str += filename;
  SQLiteDb().open(str.c_str());
  SQLiteDb().execDML("begin transaction;");
  UseSQLite() = true;
  return 1;
} // fnOpenExistingSQLite
DLLEXPORT int fnCloseSQLite ()
{
  if (!UseSQLite()) return 1;
  SQLiteDb().execDML("commit transaction;");
  SQLiteDb().close();
  return 1;
} // fnCloseSQLite
// *****************************************************************************
// NOTES:  Creates missing tables for packages where only part of the tables
//         were created.  This could occur when parameters are used for
//         a package like LPF, where none of the arrays get written
//         (everything is in parameters tables) and so that the LPFProperties
//         table never gets created.  Some of the AHGW tools need that table,
//         even if it's empty, to function properly.
//
//         Works on:  RCH, EVT, Basic, BCF, LPF
// *****************************************************************************
DLLEXPORT int fnCreateMissingTables (const char *filename)
{
  HRESULT   hr;
  CEditGDB  edit;

  edit.SetGDBFile(filename);
  hr = edit.AddMissingTables();
  if (FAILED(hr)) return 0;

  return 1;
} // fnCreateMissingTables

// *****************************************************************************
// For DisPackage1
// NOTES:  filename:  The filename of the geodatabase
//         a_vValues:  array with the data that will be written to the db
//         a_fields:  the fields of the table in the db
//         a_szTableName: the table name we are writing to
// *****************************************************************************
static int fnWriteToDB (const char *a_filename,
                        CComSafeArray<VARIANT> &a_vValues,
                        CComSafeArray<BSTR> &a_fields,
                        CComBSTR &a_szTableName)
{
  if (UseSQLite())
  {
    CEditSQLiteDb sq(&SQLiteDb());
    sq.EditTable(a_szTableName, a_fields, a_vValues);
    return 1;
  }
  else
  {
    HRESULT   hr;
    CEditGDB  edit;
    edit.SetGDBFile(a_filename);
    hr = edit.EditTable(a_szTableName, a_fields, a_vValues);
    if (FAILED(hr))
      return 0;
  }

  return 1;
} // fnWriteToDB
static int fnWriteToDbExistTable (const char *a_filename,
                                  const char *a_query,
                                  CComSafeArray<VARIANT> &a_vValues,
                                  CComSafeArray<BSTR> &a_fields,
                                  CComBSTR &a_szTableName)
{
  int nRowsEdited(0);
  const int nData(static_cast<int>(a_vValues.GetCount(1)));
  if (UseSQLite())
  {
    CEditSQLiteDb sq(&SQLiteDb());
    sq.EditTableExistingRows(a_szTableName, a_fields, a_vValues,
                             nRowsEdited, a_query);
  }
  else
  {
    HRESULT   hr;
    CEditGDB  edit;

    edit.SetGDBFile(a_filename);
    hr = edit.EditTableExistingRows(a_szTableName, a_fields, a_vValues,
                                    nRowsEdited, a_query);
    if (FAILED(hr))
      return 0;
  }
  if (0 != nRowsEdited &&
      nRowsEdited != nData)
    return 0;
  // we edited existing records so we are done
  else if (nRowsEdited == nData)
    return 1;

  return 2; // this wasn't a bad failure
} // fnWriteToDBExistTable
// *****************************************************************************
// NOTES:  Writes an array from MODFLOW to the appropriate table and field
//         in the geodb
//
//         filename:  The filename of the geodatabase
//         a_table:  The name of the db table we are editing
//         a_field:  The name of the field in the table we are writing to
//         a_nData:  The size of the array that was passed in
//         a_startCellId:  The cell id associated with the first value in the
//                         array. The array is ordered.
//         a_data:  The data array containing the values we are putting in
//                  the db.
//         a_stressPeriod: If applicable (ie RCH, EVT) this is the stress 
//                         period associated with the data.
// *****************************************************************************
template <class T>
static int fnWriteArrayT (const char *filename,
                          const char *a_table,
                          const char *a_cellField,
                          const char *a_field,
                          const char *a_query,
                          int a_nData,
                          int a_startCellId,
                          int* a_stressPeriod,
                          const T *a_data)
{
  // check if this is a valid table
  CTableDef    tableDef;
  int tableId(tableDef.GetTableIdFromString(a_table));
  if (tableId < 0)
    return 0;
  // see if the field is in the table
  if (!tableDef.FieldExistsInDef(tableId, a_field))
    return 0;
  // just package up the array data without cell ids or stress periods
  // and see if we can edit an existing table
  {
    CComSafeArray<VARIANT> vValues;
    CComSafeArray<BSTR>    fields;
    CComBSTR               szTableName;
    if (!DataToSafeArray::OnlyArrayU2DTemplate(a_table, a_field, a_nData,
                                               a_data, vValues, fields,
                                               szTableName))
      return 0;

    int rval;
    rval = fnWriteToDbExistTable(filename, a_query, vValues, fields,
                                 szTableName);
    if (0 == rval || 1 == rval)
      return rval;
  }
  // If we can't edit an existing table the add the data to the table
  {
    CComSafeArray<VARIANT> vValues;
    CComSafeArray<BSTR>    fields;
    CComBSTR               szTableName;
    if (!DataToSafeArray::U2DTemplate(a_table, a_cellField,a_field, a_nData,
                                      a_startCellId, a_stressPeriod[0],
                                      a_stressPeriod[1], a_data,
                                      vValues, fields, szTableName))
      return 0;
    return(fnWriteToDB(filename, vValues, fields, szTableName));
  }
} // fnWriteArrayT
DLLEXPORT int fnWriteArrayInt (const char *filename,
                               const char *a_table,
                               const char *a_cellField,
                               const char *a_field,
                               const char *a_query,
                               int a_nData,
                               int a_startCellId,
                               int* a_stressPeriod,
                               const int *a_data)
{
  return (fnWriteArrayT(filename, a_table, a_cellField, a_field, a_query,
                        a_nData, a_startCellId, a_stressPeriod, a_data));
} // fnWriteArrayInt
DLLEXPORT int fnWriteArrayFlt (const char *filename,
                               const char *a_table,
                               const char *a_cellField,
                               const char *a_field,
                               const char *a_query,
                               int a_nData,
                               int a_startCellId,
                               int* a_stressPeriod,
                               const float *a_data)
{
  return (fnWriteArrayT(filename, a_table, a_cellField, a_field, a_query,
                        a_nData, a_startCellId, a_stressPeriod, a_data));
} // fnWriteArrayFlt
DLLEXPORT int fnWriteArrayDbl (const char *filename,
                               const char *a_table,
                               const char *a_cellField,
                               const char *a_field,
                               const char *a_query,
                               int a_nData,
                               int a_startCellId,
                               int* a_stressPeriod,
                               const double *a_data)
{
  return (fnWriteArrayT(filename, a_table, a_cellField, a_field, a_query,
                        a_nData, a_startCellId, a_stressPeriod, a_data));
} // fnWriteArrayDbl

// *****************************************************************************
//  NOTES:  Creates a class that will store values that we will write to the
//          db once the user is done
//
//          filename:  The filename of the geodatabase
//          a_table:   the db table
//          a_query:   the query string to select the correct records
// *****************************************************************************
DLLEXPORT int fnStartCacheing (const char *filename,
                               const char *a_table,
                               const char *a_query)
{
  if (TableInfo::Exists())
    return 0;
  CStr fname(filename), tab(a_table);
  if (fname.IsEmpty())
    return 0;
  if (tab.IsEmpty())
    return 0;
  // make sure the table exists
  CTableDef tableDef;
  TableInfo::Get()->m_tabId = tableDef.GetTableIdFromString(tab);
  if (TableInfo::Get()->m_tabId < 0)
    return 0;

  TableInfo::Get()->m_fname = filename;
  TableInfo::Get()->m_query = a_query;
  TableInfo::Get()->m_tabName = a_table;
  return 1;
} // fnStartCacheing
// *****************************************************************************
//  NOTES:  This is an overload so that you can size the variant array up front
// *****************************************************************************
DLLEXPORT int fnStartCacheing2 (const char *filename,
                                const char *a_table,
                                const char *a_query,
                                int a_nFields,
                                int a_nRows)
{
  if (a_nFields < 1 ||
      a_nRows < 1)
    return 0;

  if (!fnStartCacheing(filename, a_table, a_query))
    return 0;

 // size the VARIANT array
  CComSafeArrayBound bound[2];
  bound[0].SetLowerBound(0); bound[0].SetCount(a_nFields);
  bound[1].SetLowerBound(0); bound[1].SetCount(a_nRows);
  TableInfo::Get()->m_vals.Create(bound, 2);
  return 1;
} // fnStartCacheing2
// *****************************************************************************
//  NOTES:  Writes the stored data to the db and destroys the class
// *****************************************************************************
DLLEXPORT int fnStopCacheing ()
{
  if (!TableInfo::Exists())
    return 0;
  LPSAFEARRAY *f, *v;
  f = TableInfo::Get()->m_fields.GetSafeArrayPtr();
  v = TableInfo::Get()->m_vals.GetSafeArrayPtr();
  if (!*f || !v ||
      TableInfo::Get()->m_fields.GetCount(0) == 0 ||
      TableInfo::Get()->m_vals.GetCount(0) == 0)
  {
    TableInfo::Get()->Destroy();
    return 1;
  }

  CComSafeArray<VARIANT> *valsPtr(0);

  // create a new safe variant array
  CComSafeArray<VARIANT> vals;
  if (TableInfo::Get()->m_vals.GetDimensions() < 2)
  {
    // size the VARIANT array
    UINT cnt(TableInfo::Get()->m_fields.GetCount());
    CComSafeArrayBound bound[2];
    bound[0].SetLowerBound(0); bound[0].SetCount(cnt);
    bound[1].SetLowerBound(0); bound[1].SetCount(1);
    vals.Create(bound, 2);
    long idx[2] = {0,0};
    CComVariant var;
    for (UINT i=0; i<cnt; i++)
    {
      idx[0] = i;
      var = TableInfo::Get()->m_vals.GetAt(i);
      vals.MultiDimSetAt(idx, var);
    }
    valsPtr = &vals;
  }
  else
    valsPtr = &TableInfo::Get()->m_vals;

  CComBSTR tabName = TableInfo::Get()->m_tabName;
  int rval(TableInfo::Get()->WriteToDb(valsPtr));
  TableInfo::Get()->Destroy();

  std::wstring myWString(tabName);
  std::string tmp(myWString.begin(), myWString.end());
  tmp.assign(myWString.begin(), myWString.end());
  printf("Writing to database table: %s\n", tmp.c_str());
  fflush(stdout);
  return rval;
} // fnStopCacheing
// *****************************************************************************
//  NOTES:  Stores int data and a field name
//
//          a_field:   The field name in the db table
//          a_val:     the value we are writing to the db
// *****************************************************************************
DLLEXPORT int fnAddDataToCacheInt (const char *a_field,
                                   int a_val)
{
  return iAddDataToCacheT(a_field, a_val);
} // fnAddDataToCacheInt
DLLEXPORT int fnAddDataToCacheInt2 (const char *a_field,
                                    int a_val,
                                    int row)
{
  return iAddDataToCacheT(a_field, a_val, row);
} // fnAddDataToCacheInt2
// *****************************************************************************
//  NOTES:  Stores float data and a field name
//
//          a_field:   The field name in the db table
//          a_val:     the value we are writing to the db
// *****************************************************************************
DLLEXPORT int fnAddDataToCacheFlt (const char *a_field,
                                   float a_val)
{
  return iAddDataToCacheT(a_field, a_val);
} // fnAddDataToCacheFlt
DLLEXPORT int fnAddDataToCacheFlt2 (const char *a_field,
                                    float a_val,
                                    int row)
{
  return iAddDataToCacheT(a_field, a_val, row);
} // fnAddDataToCacheFlt2
// *****************************************************************************
//  NOTES:  Stores float data and a field name
//
//          a_field:   The field name in the db table
//          a_val:     the value we are writing to the db
// *****************************************************************************
DLLEXPORT int fnAddDataToCacheDbl (const char *a_field,
                                   double a_val)
{
  return iAddDataToCacheT(a_field, a_val);
} // fnAddDataToCacheDbl
DLLEXPORT int fnAddDataToCacheDbl2 (const char *a_field,
                                    double a_val,
                                    int row)
{
  return iAddDataToCacheT(a_field, a_val, row);
} // fnAddDataToCacheDbl2
// *****************************************************************************
//  NOTES:  Stores float data and a field name
//
//          a_field:   The field name in the db table
//          a_val:     the value we are writing to the db
// *****************************************************************************
DLLEXPORT int fnAddDataToCacheStr (const char *a_field,
                                   const char *a_val)
{
  return iAddDataToCacheT(a_field, a_val);
} // fnAddDataToCacheFlt
DLLEXPORT int fnAddDataToCacheStr2 (const char *a_field,
                                    const char *a_val,
                                    int row)
{
  return iAddDataToCacheT(a_field, a_val, row);
} // fnAddDataToCacheFlt2
// *****************************************************************************
//  NOTES:  Stores float data and a field name
//
//          a_field:   The field name in the db table
//          a_val:     the value we are writing to the db
// *****************************************************************************
template <class T>
static int iAddDataToCacheT (const char *a_field,
                             T a_val,
                             int a_row/*-1*/)
{
  if (!TableInfo::Exists())
    return 0;
  // make sure the field exists in the table
  CTableDef tableDef;
  if (!tableDef.FieldExistsInDef(TableInfo::Get()->m_tabId, a_field))
    return 0;
  LPSAFEARRAY *v(TableInfo::Get()->m_vals.GetSafeArrayPtr());
  if (*v && TableInfo::Get()->m_vals.GetDimensions() > 1 &&
      a_row < 0)
    return 0;

  if (a_row < 0)
  {
    TableInfo::Get()->m_fields.Add(CComBSTR(a_field));
    TableInfo::Get()->m_vals.Add(CComVariant(a_val));
    return 1;
  }
  if (TableInfo::Get()->m_vals.GetDimensions() < 2)
    return 0;

  UINT nFields(TableInfo::Get()->m_vals.GetCount(0));
  UINT nRows(TableInfo::Get()->m_vals.GetCount(1));
  if (static_cast<UINT>(a_row) >= nRows)
    return 0;

  UINT pos(0);
  CStr str(a_field);
  std::map<CStr, UINT> *mPtr(&TableInfo::Get()->m_fieldIdx);
  if (mPtr->find(str) == mPtr->end())
  {
    LPSAFEARRAY *f(TableInfo::Get()->m_fields.GetSafeArrayPtr());
    if (*f)
      pos = TableInfo::Get()->m_fields.GetCount();
    // see if creating this field will make too many columns
    if (pos >= nFields)
      return 0;

    mPtr->insert(std::make_pair(str, pos));
    TableInfo::Get()->m_fields.Add(CComBSTR(a_field));
  }
  else
    pos = (*mPtr)[str];

  long idx[2] = {pos,a_row};
  if (S_OK != TableInfo::Get()->m_vals.MultiDimSetAt(idx, CComVariant(a_val)))
    return 0;
  return 1;
} // iAddDataToCacheT
// *****************************************************************************
//  NOTES:  Adds an AUX column to a table
// *****************************************************************************
DLLEXPORT int fnAddAuxColToTable (const char *a_table,
                                  const char *a_colName)
{
  CTableDef tabDef;
  CStr t(a_table), col(a_colName);
  tabDef.AddAuxColumnToTableDef(t, col);
  return 1;
} // fnAddAuxColToTable
// *****************************************************************************
//  NOTES:  
// *****************************************************************************
DLLEXPORT int fnDeleteAllRowsInTable (const char *a_filename,
                                      const char *a_table)
{
  if (UseSQLite())
  {
    CEditSQLiteDb sq(&SQLiteDb());
    sq.DeleteTable(a_table);
  }
  else
  {
    CEditGDB  edit;
    edit.SetGDBFile(a_filename);
    edit.DeleteTable(a_table);
  }
  return 1;
} // fnDeleteAllRowsInTable
// *****************************************************************************
//  NOTES:  Writes the data to the data base
// *****************************************************************************
int TableInfo::WriteToDb (CComSafeArray<VARIANT> *valsPtr)
{
  // write to existing table
  int rval(2);
  if (!TableInfo::Get()->m_query.IsEmpty())
  {
    rval = fnWriteToDbExistTable(TableInfo::Get()->m_fname.c_str(),
                                 TableInfo::Get()->m_query.c_str(),
                                 *valsPtr,
                                 TableInfo::Get()->m_fields,
                                 TableInfo::Get()->m_tabName);
  }
  if (2 == rval) // write new data to a table
  {
    rval = fnWriteToDB(TableInfo::Get()->m_fname.c_str(),
                       *valsPtr,
                       TableInfo::Get()->m_fields,
                       TableInfo::Get()->m_tabName);
  }
  return rval;
} // TableInfo::WriteToDb
#ifdef _DEBUG
// *****************************************************************************
//  NOTES:  Used for debugging/unit tests only.  Do not use otherwise.
//          Currently, this function writes out a row in the DISVars table, but
//          not for every field.  It then goes back to the existing rows,
//          and tries to add values for the missing fields.
//
//          filename:  The filename of the geodatabase
// *****************************************************************************
DLLEXPORT int fnDebugAppend (const char *filename)
{
  // Test out writing partially to the DISVars table,
  // then append to it
#if 0
  HRESULT   hr;
  CEditGDB  edit;

  VEC_VARIANT nlay;
  VEC_VARIANT nrow;
  VEC_VARIANT ncol;
  VEC_VARIANT itmuni;
  VEC_VARIANT lenuni;
  VEC_VARIANT reftime;
  CComVariant tempDate = __DATE__;
  tempDate.ChangeType(VT_DATE);

  // Needed for calling the edit function
  CComBSTR  szTableName;
  VEC_BSTR  vFieldNames;
  VEC_BSTR  vUsedFields;
  VEC_FTYPE vFieldTypes;
  VEC_VAR2D vValues;

  nlay.push_back(77);
  nrow.push_back(1001);
  ncol.push_back(2001);
  itmuni.push_back(123);
  lenuni.push_back(456);
  reftime.push_back(tempDate);

  // Only do the the number of layers, rows, and colums.  Do the rest later.
  vValues.clear();
  vValues.push_back(nlay);
  vValues.push_back(nrow);
  vValues.push_back(ncol);
  //vValues.push_back(itmuni);
  //vValues.push_back(lenuni);
  //vValues.push_back(reftime);
  
  // Get the table name and field names
  CTableDef TableDef;
  TableDef.GetData(DISVARS_TABLE, vFieldNames, vFieldTypes, szTableName);

  // We aren't writing to all of the fields, so just pass in the names of the
  // ones that we are using
  vUsedFields.clear();
  vUsedFields.push_back("NLAY");
  vUsedFields.push_back("NROW");
  vUsedFields.push_back("NCOL");

  edit.SetGDBFile(filename);
  // TODO
  //hr = edit.EditTable(szTableName, vUsedFields, vValues);

  // If we had an invalid argument (filename, i.e. file doesn't exist)
  // create a new geodatabase, and try again
  if (hr == E_INVALIDARG) {
    CCreateGDB create;
    CEditGDB   edit2;

    // Create a new geodatabase with all the tables
    create.SetGDBFile2(filename);
    create.SetupNewGDB(true);

    edit2.SetGDBFile(filename);
    // TODO
    //hr = edit2.EditTable(szTableName, vFieldNames, vValues);
  }

  if (FAILED(hr))
    return 0;

  // ===========================================================================
  // Now, go back and see if we can append the units and reftime fields
  vValues.clear();
  //vValues.push_back(nlay);
  //vValues.push_back(nrow);
  //vValues.push_back(ncol);
  vValues.push_back(itmuni);
  vValues.push_back(lenuni);
  vValues.push_back(reftime);

  vUsedFields.clear();
  vUsedFields.push_back("ITMUNI");
  vUsedFields.push_back("LENUNI");
  vUsedFields.push_back("RefTime");

  // Update fields in the first row of the table.... probably not what we
  // just edited, because of other unit tests
  int numedited;

  //TODO
  //hr = edit.EditTableExistingRows(szTableName, vUsedFields, vValues, numedited);

  if (FAILED(hr))
    return 0;

  // Update fields in the row we created earlier in the function.... add a
  // query filter so we only deal with rows that have "NLAY = 77"
  CComBSTR whereclause = "NLAY = 77";
  //hr = edit.EditTableExistingRows(szTableName, vUsedFields, vValues, numedited,
  //                                whereclause);

  if (FAILED(hr))
    return 0;
#endif
  return 1;
}
#endif

#ifdef CXX_TEST

#include<ModflowLink.t.h>

//------------------------------------------------------------------------------
void ModflowLinkT::testDummyTest ()
{
}
//------------------------------------------------------------------------------
void ModflowLinkT::testfnStartCacheing ()
{
  CStr fname, table, query;
  TableInfo::Get();
  // a class already exists
  TS_ASSERT(!fnStartCacheing(fname,table,query));
  TableInfo::Get()->Destroy();
  // filename is empty
  TS_ASSERT(!fnStartCacheing(fname,table,query));
  TableInfo::Get()->Destroy();
  // table name is empty
  fname = "file.mdb";
  TS_ASSERT(!fnStartCacheing(fname,table,query));
  TableInfo::Get()->Destroy();
  // table name is bogus
  table = "crap";
  TS_ASSERT(!fnStartCacheing(fname,table,query));
  TableInfo::Get()->Destroy();
  // this should work
  table = "LPFVars";
  TS_ASSERT(fnStartCacheing(fname,table,query));
  TableInfo::Get()->Destroy();
}
//------------------------------------------------------------------------------
void ModflowLinkT::testfnStartCacheing2 ()
{
  CStr fname("file.mdb"), table("LPFVars"), query;
  int nField(5), nRow(4);

  // nField 0
  TS_ASSERT(!fnStartCacheing2(fname,table,query,0,nRow));
  TS_ASSERT(!fnStartCacheing2(fname,table,query,nField,0));
  // empty file/table
  TS_ASSERT(!fnStartCacheing2("",table,query,nField,0));
  TS_ASSERT(!fnStartCacheing2(fname,"",query,nField,0));
  TableInfo::Get()->Destroy();
  // this should work
  TS_ASSERT(!fnStartCacheing2(fname,table,query,nField,0));
  TableInfo::Get()->Destroy();
}
//------------------------------------------------------------------------------
void ModflowLinkT::testfnStopCacheing ()
{
  TS_ASSERT(!fnStopCacheing());
  CStr fname("file.mdb"), table("LPFVars"), query;
  TS_ASSERT(fnStartCacheing(fname, table, query));
  TS_ASSERT(fnStopCacheing());

  TableInfoTester::MakeOne();
  fnAddDataToCacheInt("IWETIT", 1);
  TS_ASSERT(fnStopCacheing());
}
//------------------------------------------------------------------------------
void ModflowLinkT::testiAddDataToCacheT ()
{
  CStr fname("file.mdb"), table("LPFVars"), query;
  TS_ASSERT(fnStartCacheing(fname,table,query));

  int iVal(1);
  // test field doesn't exist in table
  TS_ASSERT(!iAddDataToCacheT("stuff", iVal));
  TableInfo::Get()->Destroy();

  TS_ASSERT(fnStartCacheing2(fname,table,query,1,2));
  // fails, you need to pass the row > -1
  TS_ASSERT(!iAddDataToCacheT("IWETIT", iVal));
  TS_ASSERT(iAddDataToCacheT("IWETIT", iVal, 0));
  // test too many fields for array size
  TS_ASSERT(!iAddDataToCacheT("IHDWET", iVal, 0));
  TableInfo::Get()->Destroy();
}

#endif

