#pragma once

#include <shared/dataio/dataios.h>
#include <shared/exceptions/custom_message_exception.h>

template <class T>
class MappedArray
{
public:
  MappedArray();
  MappedArray(int a_size, T a_val);
  ~MappedArray();

  void Close();

  size_t GetSize() const;

  void Init(size_t a_size, T a_val);

  T & operator [] (int a_index);

private:
  MappedArray(const MappedArray &a_rhs) { }
  bool   m_valid;
  size_t m_size;
  CFile m_fHandle;
  HANDLE m_mHandle;
  T *m_ptr;
};

//----------------------------------------------------------------------------
// Class MappedArray
//----------------------------------------------------------------------------
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::MappedArray
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
MappedArray<T>::MappedArray () :
m_valid(false),
m_size(0)
{
} // MappedArray::MappedArray
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::MappedArray
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
MappedArray<T>::MappedArray (int a_size, T a_val)
{
  m_valid = false;
  Init(a_size, a_val);
} // MappedArray::MappedArray
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::~MappedArray
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
MappedArray<T>::~MappedArray ()
{
  Close();
} // MappedArray::~MappedArray
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::Close
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
void MappedArray<T>::Close ()
{
  if (m_valid)
  {
    UnmapViewOfFile(m_mHandle);
    CloseHandle(m_mHandle);
    m_valid = false;
  }
} // MappedArray::Close
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::GetSize
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
size_t MappedArray<T>::GetSize () const
{
  return m_size;
} // MappedArray::GetSize
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::Init
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
void MappedArray<T>::Init (size_t a_size, T a_val)
{
  if (!m_valid || a_size != m_size) {
    if (m_valid) {
      Close();
    }
    m_size = a_size;

    CStr  fileName;
    daGetTempFileName(fileName);

    if (!m_fHandle.Open(fileName, CFile::modeCreate | CFile::modeReadWrite |
                             CFile::shareExclusive | CFile::osRandomAccess)) {
      throw CustomMessageException("Unable to create file for mapping.");
    }

    DWORD fptr = SetFilePointer(m_fHandle, 1000, NULL, FILE_BEGIN);
    if (fptr == INVALID_SET_FILE_POINTER) {
      throw CustomMessageException("Unable to setup file mapping -- SetFilePointer");
    }
    
    if (!SetEndOfFile(m_fHandle)) {
      throw CustomMessageException("Unable to setup file mapping -- Set End of File");
    }

    m_mHandle = CreateFileMappingA(m_fHandle, NULL, PAGE_READWRITE, NULL, 
                                        GetSize() * sizeof(T), NULL);
    if (m_mHandle == NULL) {
      throw CustomMessageException("Unable to setup file mapping -- CreateFileMapping");
    }

    m_ptr = (T*)MapViewOfFile(m_mHandle, FILE_MAP_ALL_ACCESS, 0, 0, 
                               GetSize() * sizeof(T));
    if (m_ptr == NULL) {
      throw CustomMessageException("Unable to setup file mapping -- MapViewOfFile");
    }
  }
  for (size_t i = 0; i < m_size; i++) {
    m_ptr[i] = a_val;
  }
  m_valid = true;
} // MappedArray::Init
// ---------------------------------------------------------------------------
// FUNCTION  MappedArray::operator []
// PURPOSE   
// NOTES     
// ---------------------------------------------------------------------------
template <class T>
T & MappedArray<T>::operator [] (int a_index)
{
  return m_ptr[a_index];
} // MappedArray::operator []