#include "util.h"

#include <vector>
#include <math.h>
#include "StdString.h"


void util::NullFuncArg (const char *a_,
                        int a_line)
{
  CStr file;
  util::StripPathFromFilename(a_, file);
  CStr str;
  str.Format("\nNull paramter passed to method.\nFile:%s\nLine:%d\n",
             file.c_str(),a_line);
  //ErrorStack::Get().PutError(str.c_str());
} // util::NullFuncArg
///////////////////////////////////////////////////////////////////////////////
/// \brief Fills in the string with the path to the bin directory
///////////////////////////////////////////////////////////////////////////////
void util::GetBinDirectory (CStr &a_)
{
  char c[5000];
  GetModuleFileName(NULL, c, 5000);
  CStr str(c);
  int pos(str.ReverseFind("\\"));
  a_ = str.Left(str.GetLength() - (str.GetLength() - pos));
}
///////////////////////////////////////////////////////////////////////////////
/// \brief Fills in the string with the path to the temp files directory
///////////////////////////////////////////////////////////////////////////////
void util::GetTempDirectory (CStr &a_)
{
  char *c = getenv("TEMP");
  a_ = c;
  if (a_.IsEmpty())
  {
    c = getenv("TMP");
    a_ = c;
  }
} // util::GetTempDirectory
///////////////////////////////////////////////////////////////////////////////
/// \brief Fills in the string with the path to the test files directory
///////////////////////////////////////////////////////////////////////////////
void util::GetTestFilesDirectory (CStr &a_)
{
  util::GetBinDirectory(a_);
  a_ += "\\TestFiles";
}
///////////////////////////////////////////////////////////////////////////////
/// \brief Strips the file from a file name with path so that you get back
/// the path.
///////////////////////////////////////////////////////////////////////////////
void util::StripFileFromFilename (const char *a_fWithPath,
                                  CStr &a_path)
{
  int i;
  CStr cpathandfile(a_fWithPath);

  i = cpathandfile.ReverseFind('\\');
  if (i == -1) {
    i = cpathandfile.ReverseFind('/');
    if (i == -1)
      a_path= "";
  }

  a_path = cpathandfile.Left(i+1); // we'd use i+1 if we wanted the last '\'
} // util::StripFileFromFilename
///////////////////////////////////////////////////////////////////////////////
/// \brief Strips the path from the file name.
///////////////////////////////////////////////////////////////////////////////
void util::StripPathFromFilename (const char *a_fWithPath,
                                  CStr &a_file)
{
  CStr filename(a_fWithPath);
  int i;

  i = filename.ReverseFind('\\');
  if (i == -1) {
    i = filename.ReverseFind('/');
    if (i == -1)
      a_file = a_fWithPath;
  }
  a_file = filename.Right(filename.GetLength() - i - 1);
} // util::StripPathFromFilename
///////////////////////////////////////////////////////////////////////////////
/// \brief Strips everything except the extension.
///////////////////////////////////////////////////////////////////////////////
void util::StripAllButExtension (const char *a_fWithPath,
                                 CStr &a_ext)
{
  int i;
  CStr filename(a_fWithPath);

  // Find the last period
  i = filename.ReverseFind('.');
  if (i != -1) {
    filename = filename.Right(filename.GetLength() - i - 1);
  }
  else {
    filename = "";
  }
  a_ext = filename;
} // util::StripAllButExtension
///////////////////////////////////////////////////////////////////////////////
/// \brief Deletes the directory that was passed in.
///////////////////////////////////////////////////////////////////////////////
bool util::DeleteDir (const char *a_)
{
#ifdef CXX_TEST
  int len;
  len = (int)strlen(a_);
  std::vector<char> ch(len+2);
  strcpy(&ch[0], a_);
  ch.back() = '\0';
  SHFILEOPSTRUCT fileop;
  fileop.hwnd   = NULL;    // no status display
  fileop.wFunc  = FO_DELETE;  // delete operation
  fileop.pFrom  = &ch[0];  // source file name as double null terminated string
  fileop.pTo    = NULL;    // no destination needed
  fileop.fFlags = FOF_NOCONFIRMATION|FOF_SILENT;  // do not prompt the user
  fileop.fAnyOperationsAborted = FALSE;
  fileop.lpszProgressTitle     = NULL;
  fileop.hNameMappings         = NULL;
  return(SHFileOperation(&fileop) == 0);
#else
  return true;
#endif
} // util::DeleteDir
///////////////////////////////////////////////////////////////////////////////
/// \brief Copies the src dir that was passed in to the dest dir.
///////////////////////////////////////////////////////////////////////////////
bool util::CopyDir (const char *a_src,
                    const char *a_dest)
{
#ifdef CXX_TEST
  int len;
  len = (int)strlen(a_src);
  std::vector<char> ch(len+2);
  strcpy(&ch[0], a_src);
  ch.back() = '\0';

  len = (int)strlen(a_dest);
  std::vector<char> ch1(len+2);
  strcpy(&ch1[0], a_dest);
  ch1.back() = '\0';

  SHFILEOPSTRUCT fileop;
  fileop.hwnd   = NULL;    // no status display
  fileop.wFunc  = FO_COPY;  // copy operation
  fileop.pFrom  = &ch[0];  // source file name as double null terminated string
  fileop.pTo    = &ch1[0];    // destination directory
  fileop.fFlags = FOF_NOCONFIRMATION|FOF_SILENT;  // do not prompt the user
  fileop.fAnyOperationsAborted = FALSE;
  fileop.lpszProgressTitle     = NULL;
  fileop.hNameMappings         = NULL;
  return(SHFileOperation(&fileop) == 0);
#else
  return true;
#endif
} // util::CopyDir

