!LMG INTERFACE FOR MODFLOW 2005 PROVIDED BY FRAUNHOFER SCAI
!www.scai.fraunhofer.de
!This interface connects MODFLOW to the fast hierarchical solver library SAMG by Fraunhofer SCAI
!For more information contact samg@scai.fraunhofer.de

!Editor Peter Thum

      MODULE LMGMODULE
C        REAL   ,SAVE,POINTER  ::STOR1,STOR2,STOR3
        INTEGER,          SAVE, POINTER, DIMENSION(:)     ::IA,JA,ig
        DOUBLE PRECISION, SAVE, POINTER, DIMENSION(:) ::A,U,F
        REAL   ,SAVE,POINTER  ::DAMPLMGT,DAMPLMG
        REAL   ,SAVE,POINTER  ::HCLOSELMG,RCLOSELMG,DUPLMG,DLOWLMG     
        INTEGER,SAVE,POINTER  ::controllmg
        INTEGER,SAVE,POINTER  :: isiz1,isiz2,isiz3,isiz4       
        INTEGER,SAVE,POINTER  ::MXCYC,ICG,ioutamg,iadamplmg
        INTEGER,SAVE,POINTER  ::IHCOFADDLMG
        CHARACTER*300,        save,pointer :: samg_logfile
C1        INTEGER,          SAVE, POINTER, DIMENSION(:,:)   ::LHCH
C1        REAL,             SAVE, POINTER, DIMENSION(:)     ::HCHG
C1        INTEGER,          SAVE, POINTER, DIMENSION(:,:)   ::LRCHLMG
C1        REAL,             SAVE, POINTER, DIMENSION(:)     ::RCHG
C1        INTEGER,          SAVE, POINTER, DIMENSION(:)     ::IT1
        
      TYPE LMGTYPE
C        REAL   ,POINTER  ::STOR1,STOR2,STOR3
        INTEGER,           POINTER, DIMENSION(:)     ::IA,JA,ig
        DOUBLE PRECISION,  POINTER, DIMENSION(:) ::A,U,F
        REAL   ,POINTER  ::DAMPLMGT,DAMPLMG
        REAL   ,POINTER  ::HCLOSELMG,RCLOSELMG,DUPLMG,DLOWLMG        
        INTEGER,POINTER  ::controllmg
        INTEGER,POINTER  :: isiz1,isiz2,isiz3,isiz4        
        INTEGER,POINTER  ::MXCYC,ICG,ioutamg,iadamplmg
        INTEGER,POINTER  ::IHCOFADDLMG
        CHARACTER*300,        pointer :: samg_logfile
C1        INTEGER,           POINTER, DIMENSION(:,:)   ::LHCH
C1        REAL,              POINTER, DIMENSION(:)     ::HCHG
C1        INTEGER,           POINTER, DIMENSION(:,:)   ::LRCHLMG
C1        REAL,              POINTER, DIMENSION(:)     ::RCHG
C1        INTEGER,           POINTER, DIMENSION(:)     ::IT1

      END TYPE
      TYPE(LMGTYPE), SAVE ::LMGDAT(10)
      END MODULE LMGMODULE
      
      SUBROUTINE LMG7AR(IN,MXITER,IGRID)
C     ******************************************************************
C     ALLOCATE STORAGE FOR LMG ARRAYS AND READ LMG DATA
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      use module_exportData                                             !aq
      USE GLOBAL,   ONLY:IOUT
      USE LMGMODULE,ONLY:DAMPlmg,DAMPlmgt,DUPLMG,DLOWLMG,IADAMPLMG
      implicit none
      
      CHARACTER*200 LINE
      INTEGER :: IN,MXITER,igrid
      REAL    :: DAMPLMG1
      REAL    :: HCLOSELMG1,RCLOSELMG1     
      INTEGER :: controllmg1,MXCYC1,IHCOFADD1,IOUTAMG1,ierr  
      integer flag,ISTART,ISTOP,LLOC,ICG                                !aq
      REAL STOR1,STOR2,STOR3,R                                          !aq
C     ------------------------------------------------------------------
      
C1-------INITIALIZE
      HCLOSELMG1 = 0.0
      RCLOSELMG1 = 0.0
      CONTROLlmg1=0
      MXITER=1
      MXCYC1=1
      DAMPlmg1=1.0
      IOUTAMG1=0
      IHCOFADD1=0
     
C
C-------PRINT A MESSAGE IDENTIFYING LMG PACKAGE
      WRITE (IOUT,500)
  500 FORMAT (1X,/1X,'LMG -- ALGEBRAIC MULTI-GRID SOLUTION PACKAGE',
     &        ', VERSION 3.0, 2010/05/12')
C
C-------READ AND PRINT COMMENTS, MXITER,ITER1 AND NPCOND
      CALL URDCOM(IN,IOUT,LINE)
      if (ed_getExportData()) then                                      !aq
        LLOC=1                                                          !aq
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,MXITER,R,IOUT,IN)          !aq
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,3,STOR1,R,IOUT,IN)           !aq
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,STOR2,R,IOUT,IN)           !aq
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,STOR3,R,IOUT,IN)           !aq
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,ICG,R,IOUT,IN)             !aq
      endif                                                             !aq
      WRITE (IOUT,510)
  510 FORMAT (' STOR VARIABLES WILL BE IGNORED')

      
C-------READ MXITER,MXCYC,BCLOSE,DAMP,IOUTAMG      
      READ (IN,*) MXITER,MXCYC1,rCLOSElmg1,DAMPlmg1,IOUTAMG1
C       
      IF (MXCYC1.GT. 99) MXCYC1=99
      
      allocate(DAMPlmg,DAMPlmgt,DUPLMG,DLOWLMG,IADAMPLMG)
      IADAMPlmg = 0
      DLOWLMG=0.
      DUPLMG=1.
      damplmgt=0
      damplmg=DAMPlmg1
      call LMG7DMP(IN) 
      
      READ (IN,*,iostat=ierr)  HCLOSElmg1, CONTROLlmg1
    
      CALL LMG7AR_SUB(HCLOSElmg1, CONTROLlmg1,MXITER,MXCYC1,.false.,
     +                  rCLOSElmg1,DAMPlmg1,IOUTAMG1,IHCOFADD1,IGRID)
      
      RETURN
      END
      
      SUBROUTINE LMG7AR_SUB(HCLOSElmg1, CONTROLlmg1,MXITER,MXCYC1,
     +            readdamp,rCLOSElmg1,DAMPlmg1,IOUTAMG1,IHCOFADD1,IGRID)

C     ******************************************************************
C     ALLOCATE STORAGE FOR LMG ARRAYS AND READ LMG DATA
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      use module_exportData                                             !aq
      use module_aquaveo_data                                           !aq
      use mfLib                                                         !aq
      USE GLOBAL,   ONLY:IOUT,NCOL,NROW,NLAY
      USE LMGMODULE,ONLY:MXCYC,ICG,samg_logfile,ig,iadamplmg,
     1                   ioutamg,HCLOSELMG,RCLOSELMG,dlowLMG,dupLMG,
     2                   DAMPLMGT,controllmg,a,ia,ja,u,f,DAMPLMG,
     3                   isiz1,isiz2,isiz3,isiz4,IHCOFADDLMG  
C1                       ,LHCH,HCHG,LRCHLMG,RCHG,IT1
C
      implicit none
      
      integer flag,ISTART,ISTOP,LLOC                                    !aq
      REAL STOR1,STOR2,STOR3,R                                          !aq
      CHARACTER*200 LINE
      INTEGER IN,MXITER,ierr,nodes,nend,nbegin,itemp,igrid
      real hcloselmg1,rcloselmg1,damplmg1
      integer CONTROLlmg1,MXCYC1,IOUTAMG1,IHCOFADD1
      integer samg_logio
      logical readdamp
C     ------------------------------------------------------------------

C <
      ierr=0

      ALLOCATE(MXCYC,ICG,samg_logfile,controllmg)
      ALLOCATE(HCLOSELMG,RCLOSELMG)
      ALLOCATE(ioutamg,isiz1,isiz2,isiz3,isiz4,IHCOFADDLMG)
C   get name of logfile    
      inquire(unit=iout,name=samg_logfile)
      if (trim(samg_logfile).eq.'')
     &   stop '*** ERROR: name of LST file not defined ?! ***'
      nbegin=1
      do
         itemp=index(samg_logfile(nbegin:300),'.')
         if (itemp.eq.0) exit
         if(itemp+1+nbegin.ge.300) exit
         nbegin=itemp+1+nbegin
      enddo
      nend=nbegin-3
      nbegin=1
      if (NOT(ed_getExportData())) then                                 !aq
      if (nend.le.0) then
        stop '*** ERROR: LST file does not contain proper suffix! ***'
      endif
      endif                                                             !aq
      samg_logfile(1:nend+5)=samg_logfile(nbegin:nend)//'.samg'
      samg_logfile(nend+6:300)=''


      
C1-------INITIALIZE
      HCLOSELMG = hCLOSElmg1
      RCLOSELMG = rCLOSElmg1
      CONTROLlmg=CONTROLlmg1
      ICG=1
      IHCOFADDLMG=IHCOFADD1
      MXCYC=MXCYC1
      IOUTAMG=IOUTAMG1
      
C
C-------PRINT A MESSAGE IDENTIFYING LMG PACKAGE
      WRITE (IOUT,500)
  500 FORMAT (1X,/1X,'LMG -- ALGEBRAIC MULTI-GRID SOLUTION PACKAGE',
     &        ', VERSION 3.0, 2010/05/12')
C
C-------READ AND PRINT COMMENTS, MXITER,ITER1 AND NPCOND
C      CALL URDCOM(IN,IOUT,LINE)
C      LLOC=1
C      CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,MXITER,R,IOUT,IN)
C      CALL URWORD(LINE,LLOC,ISTART,ISTOP,3,STOR1,R,IOUT,IN)
C      CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,STOR2,R,IOUT,IN)
C      CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,STOR3,R,IOUT,IN)
C      CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,ICG,R,IOUT,IN)
      WRITE (IOUT,510)
  510 FORMAT (' STOR VARIABLES WILL BE IGNORED')
C      IF (ICG.NE.1) ICG=0   
C
C-------ALLOCATE SPACE FOR THE LMG ARRAYS
      NODES=NCOL*NROW*NLAY
      isiz1=7*NODES-2*(NROW*NLAY+NCOL*NLAY+NROW*NCOL)
      isiz2=nodes+1
      isiz3=nodes
      isiz4=nodes
      
C     EACH NODE HAS 6 NEIGHBOURS
      ALLOCATE (A(isiz1))
      ALLOCATE (JA(isiz1))
      ALLOCATE (IA(isiz2))
      ALLOCATE (F(isiz4))
      ALLOCATE (U(isiz4))
      ALLOCATE (IG(isiz3))
      
C-------READ MXITER,MXCYC,BCLOSE,DAMP,IOUTAMG      
C      READ (IN,*) MXITER,MXCYC,rCLOSElmg,DAMPlmg,IOUTAMG
C       
      IF (MXCYC.GT. 99) MXCYC=99
      IOUTAMG=3-IOUTAMG


      if (readdamp) then
         allocate(DAMPlmg,DAMPlmgt,DUPLMG,DLOWLMG,IADAMPLMG)
         IADAMPlmg = 0
         DLOWLMG=0.
         DUPLMG=1.
         damplmgt=0
         damplmg=DAMPlmg1
         CALL LMG7DMP(IN) 
      endif
      
      IF ((CONTROLlmg.LT.1.OR.CONTROLlmg.GT.3).OR.IERR.NE.0) THEN
         CONTROLlmg=0
         HCLOSElmg=rCLOSElmg
      endif
  
 
C1      ITMEM=MXITER*ITER1
C1      ALLOCATE (HCHG(ITMEM))
C1      ALLOCATE (LHCH(3,ITMEM))
C1      ALLOCATE (RCHG(ITMEM))
C1      ALLOCATE (LRCHLMG(3,ITMEM))
C1      ALLOCATE (IT1(ITMEM))     
 
      CALL LMG7PSV(IGRID)
      samg_logio=77   
      
      call LMG1RPsamg(MXITER,MXCYC,RCLOSELMG,DAMPLMG,damplmgt,
     &                  IOUTAMG,ICG,IADAMPlmg,DUPLMG,DLOWLMG,HCLOSELMG,
     &                  CONTROLLMG,samg_logio,SAMG_LOGFILE) 
C 
C
      call exp_LmgLine1(STOR1,STOR2,STOR3,ICG)                          !aq
      call exp_LmgPackage('LMG',MXITER,MXCYC,rCLOSElmg,DAMPLMG,IOUTAMG, !aq
     1                    DUPLMG,DLOWLMG,HCLOSELMG,CONTROLLMG)          !aq
      
      RETURN
      END
      
      SUBROUTINE LMG7DMP(IN)    
      USE LMGMODULE,ONLY:DAMPlmg,DAMPlmgt,DUPLMG,DLOWLMG,IADAMPLMG
      implicit none
      integer :: IN
      
C2------CHECK VALUE OF DAMP AND SEE IF ADAPTIVE DAMPING IS SELECTED.
C2------USE 1.E-6 AS A TOLERANCE WHEN CHECKING IF DAMP = -1 OR -2
C2------SET DAMP TO 1.0 FOR ALL OTHER VALUES, UNLESS THIS IS GREATER 
C2------THAN DUP, IN WHICH CASE SET DAMP TO DUP.      
      DAMPLMGT=DAMPLMG
      IF(DAMPLMG .LE. 0.)THEN
        IF(ABS(DAMPLMG + 1.0) .LT. 1.E-6)THEN
          DAMPLMG= 1.0 
          IADAMPLMG = -1
        ELSEIF(ABS(DAMPlmg + 2.0) .LT. 1.E-6)THEN
          DAMPLMG=1.0
          IADAMPLMG = -2
          READ(IN,*) DUPLMG,DLOWLMG
           IF(DUPlmg .LT. 1.0)DAMPlmg=DUPLMG
        ELSEIF(ABS(DAMPlmg + 3.0) .LT. 1.E-6)THEN
          IADAMPLMG = -3
          READ(IN,*) DAMPLMG,DAMPLMGT
        ELSE
          DAMPLMG=1.0
        ENDIF
      ENDIF      
      end SUBROUTINE
      
      SUBROUTINE LMG7PNT(IGRID)
C  Save pointers to LMG data
      USE LMGMODULE
C
      implicit none
      integer igrid
      
      MXCYC=>LMGDAT(IGRID)%MXCYC
      ICG=>LMGDAT(IGRID)%ICG
      samg_logfile=>LMGDAT(IGRID)%samg_logfile
      iadamplmg=>LMGDAT(IGRID)%iadamplmg
      controllmg=>LMGDAT(IGRID)%controllmg
      HCLOSELMG=>LMGDAT(IGRID)%HCLOSELMG
      RCLOSELMG=>LMGDAT(IGRID)%RCLOSELMG
      IHCOFADDLMG=>LMGDAT(IGRID)%IHCOFADDLMG
      dlowLMG=>LMGDAT(IGRID)%dlowLMG
      dupLMG=>LMGDAT(IGRID)%dupLMG
      DAMPLMG=>LMGDAT(IGRID)%DAMPLMG
      DAMPLMGT=>LMGDAT(IGRID)%DAMPLMGT
      ioutamg=>LMGDAT(IGRID)%ioutamg
      isiz1=>LMGDAT(IGRID)%ISIZ1
      isiz2=>LMGDAT(IGRID)%ISIZ2
      isiz3=>LMGDAT(IGRID)%ISIZ3
      isiz4=>LMGDAT(IGRID)%ISIZ4
      ig=>LMGDAT(IGRID)%IG
      ia=>LMGDAT(IGRID)%Ia
      ja=>LMGDAT(IGRID)%ja
      a=>LMGDAT(IGRID)%a
      u=>LMGDAT(IGRID)%u
      f=>LMGDAT(IGRID)%f

              
C1      LHCH=>LMGDAT(IGRID)%LHCH
C1      HCHG=>LMGDAT(IGRID)%HCHG
C1      LRCHLMG=>LMGDAT(IGRID)%LRCHLMG
C1      RCHG=>LMGDAT(IGRID)%RCHG
C1      IT1=>LMGDAT(IGRID)%IT1
C
      RETURN
      END
      
      SUBROUTINE LMG7PSV(IGRID)
C  Save pointers to LMG data
      USE LMGMODULE
C
      implicit none
      integer igrid
      
      LMGDAT(IGRID)%MXCYC=>MXCYC
      LMGDAT(IGRID)%ICG=>ICG
      LMGDAT(IGRID)%samg_logfile=>samg_logfile
      LMGDAT(IGRID)%iadamplmg=>iadamplmg
      LMGDAT(IGRID)%controllmg=>controllmg
      LMGDAT(IGRID)%HCLOSELMG=>HCLOSELMG
      LMGDAT(IGRID)%RCLOSELMG=>RCLOSELMG
      LMGDAT(IGRID)%dlowLMG=>dlowLMG
      LMGDAT(IGRID)%dupLMG=>dupLMG
      LMGDAT(IGRID)%DAMPLMG=>DAMPLMG
      LMGDAT(IGRID)%DAMPLMGT=>DAMPLMGT
      LMGDAT(IGRID)%ioutamg=>ioutamg
      LMGDAT(IGRID)%IHCOFADDLMG=>IHCOFADDLMG
      LMGDAT(IGRID)%ISIZ1=>isiz1
      LMGDAT(IGRID)%ISIZ2=>isiz2
      LMGDAT(IGRID)%ISIZ3=>isiz3
      LMGDAT(IGRID)%ISIZ4=>isiz4
      LMGDAT(IGRID)%IG=>IG
      LMGDAT(IGRID)%ia=>Ia
      LMGDAT(IGRID)%ja=>ja
      LMGDAT(IGRID)%a=>a
      LMGDAT(IGRID)%u=>u
      LMGDAT(IGRID)%f=>f            
      
C1      LMGDAT(IGRID)%LHCH=>LHCH
C1      LMGDAT(IGRID)%HCHG=>HCHG
C1      LMGDAT(IGRID)%LRCHLMG=>LRCHLMG
C1      LMGDAT(IGRID)%RCHG=>RCHG
C1      LMGDAT(IGRID)%IT1=>IT1
C
      RETURN
      END
      
C***********************************************************************
      SUBROUTINE LMG7AP(HNEW,IBOUND,CR,CC,CV,HCOF,RHS,A,IA,JA,
     &                  U,FRHS,IG,ISIZ1,ISIZ2,
     &                  ISIZ3,ISIZ4,KITER,BCLOSE,DAMP,ICNVG,
     &                  KSTP,KPER,MXITER,MXCYC,NCOL,NROW,NLAY,NODES,
     &                  HNOFLO,IOUT,IOUTAMG,ICG,IADAMP,DUP,DLOW,DAMPTR,
     &                  ISS,NITER,IHCOFADD,
     &                  start_res,end_res,iter_done,setup_done)
C

      use module_exportData                                             !aq
      implicit none
      character*300 samg_logfile
C     ------------------------------------------------------------------
      CHARACTER*1 DIGIT(0:9)
      CHARACTER*2 PSUF
      CHARACTER*11 FNAME
      REAL DUP, DLOW
      DOUBLEPRECISION HNEW,DZERO,RRHS,HHCOF,RSQ,FBAR,FMAX,DDAMP,DONE
      DOUBLEPRECISION Z,B,D,E,F,H,S
      INTEGER IZERO,NODES,ISIZ1,ISIZ2,ISIZ3,ISIZ4,ISIZ5,MXITER,MXCYC
      REAL HNOFLO,BCLOSE,HCLOSE,DAMP,damptr
      INTEGER NCOL,NROW,NLAY,IBOUND,IA,JA,IG,KITER,ICNVG,KSTP,KPER
      DOUBLEPRECISION A(ISIZ1),U(ISIZ4),FRHS(ISIZ4),EPS,ECG1,ECG2,EWT2
      DOUBLEPRECISION ZHNEW, BHNEW, DHNEW, FHNEW, HHNEW, SHNEW
      PARAMETER (DZERO=0.D0, DONE=1.D0, IZERO=0)
      REAL HCOF,RHS,CR,CV,CC
      DIMENSION  IBOUND(NODES), HNEW(NODES),CR(NODES), CC(NODES),
     1   CV(NODES), HCOF(NODES), RHS(NODES), IA(ISIZ2),
     2   JA(ISIZ1),IG(ISIZ3)
      INTEGER I,IOUT,IOUTAMG,NBEGIN,ITEMP,NEND,ICG,IADAMP,iss
      integer samg_logio,IHCOFADD,NITER
      
      integer iter_done,setup_done
      real  start_res,end_res
      integer iLicense                                                  !aq
      
C      INCLUDE 'PARALLEL.INC'
      DATA (DIGIT(I),I=0,9)/'0','1','2','3','4','5','6','7','8','9'/
C     ------------------------------------------------------------------
C
C     ------------------------------------------------------------------
C
C   get name of logfile    
      inquire(unit=iout,name=samg_logfile)
      if (trim(samg_logfile).eq.'')
     &   stop '*** ERROR: name of LST file not defined ?! ***'
      nbegin=1
      do
         itemp=index(samg_logfile(nbegin:300),'.')
         if (itemp.eq.0) exit
         if(itemp+1+nbegin.ge.300) exit
         nbegin=itemp+1+nbegin
      enddo
      nend=nbegin-3
      nbegin=1
      if (NOT(ed_getExportData())) then                                 !aq
      if (nend.le.0) then
        stop '*** ERROR: LST file does not contain proper suffix! ***'
      endif
      endif                                                             !aq
      samg_logfile(1:nend+5)=samg_logfile(nbegin:nend)//'.samg'
      samg_logfile(nend+6:300)=''
      
      IF(ISS.EQ.0.and.iadamp.eq.-3) THEN
        DAMP=DAMPTR
      END IF
      samg_logio=77
      NITER=MXCYC
      call LMG1APsamg(HNEW,IBOUND,CR,CC,CV,HCOF,RHS,A,IA,JA,
     &                  U,FRHS,IG,ISIZ1,ISIZ2,
     &                  ISIZ3,ISIZ4,KITER,BCLOSE,DAMP,ICNVG,
     &                  KSTP,KPER,MXITER,NITER,NCOL,NROW,NLAY,NODES,
     &                  HNOFLO,IOUTAMG,ICG,IADAMP,DUP,DLOW,
     &                  samg_logio,IHCOFADD,
     &                  start_res,end_res,iter_done,setup_done,
     &                  iLicense, samg_logfile)                         !aq
      if (iLicense.EQ.0)  call USTOP('Not licensed for SAMG Solver.')   !aq


      return
      end
      
      SUBROUTINE LMG7DA(IGRID)
C  Deallocate LMG DATA
      USE LMGMODULE
C
      CALL LMG7PNT(IGRID)
        DEALLOCATE(isiz1,isiz2,isiz3,isiz4)
        DEALLOCATE(DAMPLMGT,DAMPLMG,controllmg)
        DEALLOCATE(IHCOFADDLMG)
        DEALLOCATE(JA)
        DEALLOCATE(IA)
        DEALLOCATE(A)
        DEALLOCATE(IG)
        DEALLOCATE(U)
        DEALLOCATE(F)
        DEALLOCATE(HCLOSELMG,RCLOSELMG,DUPLMG,DLOWLMG)
        DEALLOCATE(MXCYC,ICG,ioutamg,iadamplmg)
        DEALLOCATE(samg_logfile)
C
      RETURN
      END   
      
